# Generated by roxygen2: do not edit by hand

S3method(print,data_nugget)
export(ABCdeep.SingleInMultiple)
export(ABCdist.SingleInMultiple)
export(ABCpp.SingleInMultiple)
export(ADC)
export(ADEC)
export(BinFeaturesPlot_MultipleData)
export(BinFeaturesPlot_SingleData)
export(BoxPlotDistance)
export(CEC)
export(CVAA)
export(CharacteristicFeatures)
export(ChooseCluster)
export(Cluster)
export(ClusterCols)
export(ClusterPlot)
export(ClusteringAggregation)
export(ColorPalette)
export(ColorsNames)
export(CompareInteractive)
export(ComparePlot)
export(CompareSilCluster)
export(CompareSvsM)
export(ConsensusClustering)
export(ContFeaturesPlot)
export(Cyclogram)
export(DetermineWeight_SilClust)
export(DetermineWeight_SimClust)
export(DiffGenes)
export(DiffGenesSelection)
export(Distance)
export(Distance_v2)
export(EHC)
export(EnsembleClustering)
export(EvidenceAccumulation)
export(FeatSelection)
export(FeaturesOfCluster)
export(FindCluster)
export(FindElement)
export(FindGenes)
export(Geneset.intersect)
export(Geneset.intersectSelection)
export(HBGF)
export(HeatmapPlot)
export(HeatmapSelection)
export(HierarchicalEnsembleClustering)
export(LUCID)
export(LabelCols)
export(LabelPlot)
export(LinkBasedClustering)
export(M_ABCdeep)
export(M_ABCdist)
export(M_ABCdist.WC)
export(M_ABCpp)
export(NEMO)
export(Normalization)
export(PathwayAnalysis)
export(Pathways)
export(PathwaysIter)
export(PathwaysSelection)
export(PlotPathways)
export(PreparePathway)
export(ProfilePlot)
export(ReorderToReference)
export(SNF)
export(SelectnrClusters)
export(SharedComps)
export(SharedGenesPathsFeat)
export(SharedSelection)
export(SharedSelectionLimma)
export(SharedSelectionMLP)
export(SimilarityHeatmap)
export(SimilarityMeasure)
export(TrackCluster)
export(WeightedClust)
export(Whclust)
export(Wkmeans)
export(WonM)
export(characterize_clusters)
export(cluster_agreement)
export(cluster_biology_panel)
export(cluster_k_sweep)
export(cluster_selection_plot)
export(clustering_concordance)
export(compare_clusterings)
export(compare_methods_plot)
export(create_data_nuggets)
export(embedding_plot)
export(f.clustABC.MultiSource)
export(intNMF)
export(mosaic_labels)
export(mosaic_sim)
export(nugget_cluster)
export(nugget_feature_weights)
export(plot_cluster_profile)
export(simulate_weighting_regime)
export(spectral_clustering)
import(cluster)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(Rdpack,reprompt)
importFrom(matrixStats,colVars)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
useDynLib(MosaiClusteR, .registration = TRUE)
