# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mcpp_rf_sample <- function(populationSize, sampleSize, useWeights = FALSE, withReplacement = TRUE, weights = NULL) {
    .Call(`_MosaiClusteR_mcpp_rf_sample`, populationSize, sampleSize, useWeights, withReplacement, weights)
}

mcpp_sample_weighted_no_replace <- function(z, ng) {
    .Call(`_MosaiClusteR_mcpp_sample_weighted_no_replace`, z, ng)
}

count_coclustering_cpp <- function(res) {
    .Call(`_MosaiClusteR_count_coclustering_cpp`, res)
}

