% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-characterize.r
\name{TrackCluster}
\alias{TrackCluster}
\title{Track a cluster or a selection of objects across multiple methods}
\usage{
TrackCluster(
  List,
  Selection,
  nrclusters = NULL,
  followMaxComps = FALSE,
  followClust = TRUE,
  fusionsLog = TRUE,
  weightclust = TRUE,
  names = NULL,
  selectionPlot = FALSE,
  table = FALSE,
  completeSelectionPlot = FALSE,
  ClusterPlot = FALSE,
  cols = NULL,
  legendposx = 0.5,
  legendposy = 2.4,
  plottype = "sweave",
  location = NULL
)
}
\arguments{
\item{List}{A list of clustering outputs to be compared. The first element of
the list is used as the reference in \code{ReorderToReference}.}

\item{Selection}{Either a character vector of objects, or a numeric cluster number.}

\item{nrclusters}{The number of clusters to cut the dendrogram in. Default is NULL.}

\item{followMaxComps}{Logical. Whether to follow the cluster with the maximum
number of objects. Default is FALSE.}

\item{followClust}{Logical. Whether to follow the cluster of interest. Default is TRUE.}

\item{fusionsLog}{Logical. Indicator for the fusion of clusters. Default is TRUE.}

\item{weightclust}{Logical. For the outputs of CEC, WeightedClust or
WeightedSimClust, if TRUE only the result of the Clust element is considered.
Default is TRUE.}

\item{names}{Optional. Names of the methods. Default is NULL.}

\item{selectionPlot}{Logical. Whether to produce the selection plot. Default is FALSE.}

\item{table}{Logical. Whether to produce a table of the tracking. Default is FALSE.}

\item{completeSelectionPlot}{Logical. Whether to produce the complete selection
plot. Default is FALSE.}

\item{ClusterPlot}{Logical. Whether to produce the cluster plot. Default is FALSE.}

\item{cols}{The colors to use in the plots. Default is NULL.}

\item{legendposx}{The x-coordinate of the legend. Default is 0.5.}

\item{legendposy}{The y-coordinate of the legend. Default is 2.4.}

\item{plottype}{Should be one of "pdf","new" or "sweave". Default is "sweave".}

\item{location}{If plottype is "pdf", a location should be provided. Default is NULL.}
}
\value{
A list with one element per method describing how the selection is
distributed across the clusters, and optionally a tracking table.
}
\description{
\code{TrackCluster} follows a selection of objects or a specific cluster
across a sequence of clustering results, recording how the objects are
distributed over the clusters of each method and optionally producing
tracking plots.
}
\examples{
\dontrun{
data(fingerprintMat)
data(targetMat)

MCF7_F = Cluster(fingerprintMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)
MCF7_T = Cluster(targetMat,type="data",distmeasure="tanimoto",normalize=FALSE,
method=NULL,clust="agnes",linkage="flexible",gap=FALSE,maxK=55,StopRange=FALSE)

L=list(MCF7_F,MCF7_T)
MCF7_Track=TrackCluster(List=L,Selection=4,nrclusters=7,followMaxComps=FALSE,
followClust=TRUE,selectionPlot=TRUE,table=TRUE,cols=NULL)
}
}
