% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-navigation.r
\name{FindElement}
\alias{FindElement}
\title{Find an element in a data structure}
\usage{
FindElement(what = NULL, object = NULL, element = list())
}
\arguments{
\item{what}{A character string indicating which object to look for. Default is NULL.}

\item{object}{The data structure to look into. Only the classes data frame
and list are supported. Default is NULL.}

\item{element}{Not to be specified by the user.}
}
\value{
The returned value is a list with an element for each object found.
The element contains everything the object contained in the original data
structure.
}
\description{
The function \code{FindElement} is used internally in the
\code{PreparePathway} function but might come in handy for other uses as well.
Given the name of an object, the function searches for that object in the data
structure and extracts it. When multiple objects have the same name, all are
extracted.
}
\examples{
\dontrun{
obj <- list(a = 1, b = list(TopDE = 42, c = list(TopDE = 7)))
FindElement(what = "TopDE", object = obj)
}
}
